# 代码法
import numpy as np
import pandas as pd

np.random.seed(2)
# 产生2500个随机数，随机数区间属于0-10000
rand = np.random.randint(0, 10000, 2500)
# 将抽取的随机数划分等级
m = []
for x in rand:
    if x <= 1256:
        m.append('D')
    else:
        if 1257 <= x <= 3072:
            m.append('E')
        else:
            if 3073 <= x <= 4841:
                m.append('F')
            else:
                if 4842 <= x <= 6934:
                    m.append('G')
                else:
                    if 6935 <= x <= 8473:
                        m.append('H')
                    else:
                        if 8474 <= x <= 9478:
                            m.append('I')
                        else:
                            m.append('J')
# 将其转换为数据框格式
m = pd.DataFrame(m)
# 计算各种颜色需要抽取的个数
m.value_counts()
# 导入数据
data = pd.read_csv("C:\\Users\\lenovo\\Desktop\\diamonds.csv")  # reading file
print(data)
# 将抽取的标签和个数存入字典中,作为分层抽样的依据
diamondsdict = {'G': 514, 'F': 451, 'E': 426, 'H': 412, 'D': 300, 'I': 278, 'J': 119}
gbr = data.groupby('color')
gbr.groups


def diamondsamling(group, diamondsdict):  # 定义分层抽样的函数
    name = group.name
    n = diamondsdict[name]
    return group.sample(n=n)


result = data.groupby(
    'color', group_keys=False
).apply(diamondsamling, diamondsdict)  # 进行抽样

# 方法2 拉希里法
data['color'].value_counts()  # 输出每种颜色的个数
# 将其存入字典当中
diamondsdict = {1: 6775, 2: 9797, 3: 9542, 4: 11292, 5: 8304, 6: 5422, 7: 2808}
m *= 11292  # 得到m*=max（mi）
N = 7

alllist = []  # 该列表用来存储所抽取的群是哪个
np.random.seed(42)
for i in range(1, 10000000000):
    a = np.random.randint(1, 11293)
    b = np.random.randint(1, 7)
    s = diamondsdict.get(b)
    if len(alllist) == 2500:  # 当列表长度=2500时，停止循环
        break
    else:
        if a > s:
            alllist.append(b)

# 将其转换为数据框格式
alllist = pd.DataFrame(alllist)
# 计算各种颜色需要抽取的个数
alllist.value_counts()
# 利用选取的个数进行抽样
diamondsdict = {'D': 689, 'E': 236, 'F': 250, 'G': 0, 'H': 462, 'I': 863, 'J': 0}
gbr = data.groupby('color')
gbr.groups


def diamondsamling(group, diamondsdict):  # 定义分层抽样的函数
    name = group.name
    n = diamondsdict[name]
    return group.sample(n=n)


np.random.seed(42)
result = data.groupby(
    'color', group_keys=False
).apply(diamondsamling, diamondsdict)  # 进行抽样



import random


def beta_s(w, a, b):  # beta分布
    return w ** (a - 1) * (1 - w) ** (b - 1)


def random_coin(p):
    unif = random.uniform(0, 1)
    if unif >= p:
        return False
    else:
        return True


# 定义稳定于beta分布的MCMC函数
def beta_mcmc(N_hops, a, b):
    states = []
    cur = random.uniform(0, 1)
    for i in range(0, N_hops):
        states.append(cur)
        next = random.uniform(0, 1)
        ap = min(beta_s(next, a, b) / beta_s(cur, a, b), 1)  # 计算接受概率
        if random_coin(ap):
            cur = next
    return states[-1000:]  # 返回马尔科夫链上最近的1000个状态点


# 画图
import numpy as np
from matplotlib import pylab as plt
import scipy.special as ss

plt.rcParams['figure.figsize'] = (17.0, 4.0)


# 定义函数
def beta(a, b, i):
    e1 = ss.gamma(a + b)
    e2 = ss.gamma(a)
    e3 = ss.gamma(b)
    e4 = i ** (a - 1)
    e5 = (1 - i) ** (b - 1)
    return (e1 / (e2 * e3)) * e4 * e5


# 画图
def plot_beta(a, b):
    Ly = []
    Lx = []
    i_list = np.mgrid[0:1:100j]
    for i in i_list:
        Lx.append(i)
        Ly.append(beta(a, b, i))
    plt.plot(Lx, Ly, label="Real Distribution: a=" + str(a) + ", b=" + str(b))
    plt.hist(beta_mcmc(100000, a, b), density=True, bins=25,
             histtype='step', label="Simulated_MCMC: a=" + str(a) + ", b=" + str(b))
    plt.legend()
    plt.show()


plot_beta(0.1, 0.1)
plot_beta(1, 1)
plot_beta(2, 3)

from scipy.stats import norm


def norm_dist_prob(theta):
    y = norm.pdf(theta, loc=3, scale=2)
    return y


T = 5000
pi = [0 for i in range(T)]
sigma = 1
t = 0
while t < T - 1:
    t = t + 1
    pi_star = norm.rvs(loc=pi[t - 1], scale=sigma, size=1, random_state=None)  # 状态转移进行随机抽样
    alpha = min(1, (norm_dist_prob(pi_star[0]) / norm_dist_prob(pi[t - 1])))  # alpha值

    u = random.uniform(0, 1)
    if u < alpha:
        pi[t] = pi_star[0]
    else:
        pi[t] = pi[t - 1]

plt.scatter(pi, norm.pdf(pi, loc=3, scale=2), label='Target Distribution')
num_bins = 50
plt.hist(pi, num_bins, density=1, facecolor='red', alpha=0.7, label='Samples Distribution')
plt.legend()
plt.show()


#枪手对决
function A=Leo(A,p1,p2,p3)
if A(1,1)==1&rand<p1
    if A(2,1)==1
        A(2,1)=0;
    elseif A(3,1)==1
        A(3,1)=0;
    else
    end
end
end


function A=Raph(A,p1,p2,p3)
if A(2,1)==1&rand<p2
    if A(1,1)==1
        A(1,1)=0;
    elseif A(3,1)==1
        A(3,1)=0;
    else
    end
end
end

#最弱的Mikey ，反而有很强的能动性 Mikey不放空枪的情况：

function A=Mikey(A,p1,p2,p3)
if A(3,1)==1&rand<p3
    if A(1,1)==1&A(2,1)==0
        A(1,1)=0;
    elseif A(1,1)==0&A(2,1)==1
        A(2,1)=0;
    elseif A(1,1)==1&A(2,1)==1
        A(1,1)=0;
    end
end
end

#如果Leo和Raph都没死，那么Mikey故意放空枪：

function A=Mikey(A,p1,p2,p3)
if A(3,1)==1&rand<p3
    if A(1,1)==1&A(2,1)==0
        A(1,1)=0;
    elseif A(1,1)==0&A(2,1)==1
        A(2,1)=0;
    elseif A(1,1)==1&A(2,1)==1
        A=A;
    end
end
end

#构建决斗场：

while sum(A)>=2%
    A=Leo(A,p1,p2,p3);
    A=Raph(A,p1,p2,p3);
    A=Mikey(A,p1,p2,p3);
end

#下面的程序把这场决斗重复一万次，并对结果进行计数

function C=main3()

B=[0;0;0];
for i=1:10000
    B=B+dual();
end
C=B./sum(B);
end




import setuptools

setuptools.setup(
    name='group-11-project', #库的名字
    version='0.0.11',#库的版本号，后续更新的时候只需要改版本号
    author='lyyyyyyy',#名字
    author_email='liangying0354@163.com',#邮箱
    description='第11小组抽样设计',#介绍
    long_descroption_content_type='text/markdown',
    url='https://github.com/',
    packages=setuptools.find_packages(),
    cassifiers=[
        'Programming language:: Python ::3',
        'License :: OSI Approved :: MIT License',
        'OPerating System :: OS Independent'
    ],
)



#不用软件自带的包 基于简单随机抽样对钻石价格-重量进行分析
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import warnings
warnings.filterwarnings("ignore")
data = pd.read_csv(r'C:\Users\Admin\Desktop\diamonds.csv')


''' 惰性求值    yield  
即：你需要时才获取，包含yield语句得函数可以用来创建生成器对象，这样得函数也称生成器函数，每次执行到yiled语句会返回一个值然后暂停或挂起后面得代码得执行，下次通过生成器对象得__next()__方法，内置函数x.next()，for 循环遍历生成器对象元素或其他方式显示“索要”数据恢复执行。'''

from random import randint
import time

def myrandint( start,end,seed=999999999 ):
    a=32310901
    b=1729
    rOld=seed   #将种子seed赋值给rOld
    m=end-start   #得到m 模数
    while True:
        rNew=int(( a*rOld+b )%m)   #开始产生随机数
        #遇到yield关键字暂时挂起后面的代码，等带next(r)的调用并返回 rNew
        yield rNew
    rOld=rNew

for i in range(500):
    now=time.time()+randint(0,5000)     #时间戳加一个随机数作为种子
    print(now)
    r=myrandint(1,10000,now)     #把时间种子作为参数调用myrandint函数
    print( "种子",now,"生成的随机数:" )
    for j in range(500):
        print( next(r),end="," )
    print()

price = data.iloc[:, 0]
carat = data.iloc[:, 1]

# price = beta_0 + beta_1 * carat + epsilon

def func(carat, beta_0, beta_1, epsilon):
    price = beta_0 + beta_1 * carat + epsilon
    return price

popt, pcov = curve_fit(func, carat, price)
dat = []
for i in range(0, 500):
    data1 = data.sample(n=5000)
    price = data1.iloc[:, 0]
    carat = data1.iloc[:, 1]
    popt, pcov = curve_fit(func, carat, price)
dat.append(popt)

dat = np.array(dat)
mean = np.average(dat[:,1], axis=0)
var = np.var(dat[:,1], axis=0)
std = np.std(dat[:,1], axis=0)
